@@ -21,7 +21,7 @@ from borg.logger import setup_logging  # noqa: E402
 # Ensure that the loggers exist for all tests
 setup_logging()
 
-from borg.testsuite import has_lchflags, has_llfuse  # noqa: E402
+from borg.testsuite import has_lchflags, has_llfuse, has_pyfuse3  # noqa: E402
 from borg.testsuite import are_symlinks_supported, are_hardlinks_supported, is_utime_fully_supported  # noqa: E402
 from borg.testsuite.platform import fakeroot_detected  # noqa: E402
 
@@ -32,7 +32,8 @@ def clean_env(tmpdir_factory, monkeypatch):
     monkeypatch.setenv('XDG_CONFIG_HOME', str(tmpdir_factory.mktemp('xdg-config-home')))
     monkeypatch.setenv('XDG_CACHE_HOME', str(tmpdir_factory.mktemp('xdg-cache-home')))
     # also avoid to use anything from the outside environment:
-    keys = [key for key in os.environ if key.startswith('BORG_')]
+    keys = [key for key in os.environ
+            if key.startswith('BORG_') and key not in ('BORG_FUSE_IMPL', )]
     for key in keys:
         monkeypatch.delenv(key, raising=False)
 
@@ -40,7 +41,8 @@ def clean_env(tmpdir_factory, monkeypatch):
 def pytest_report_header(config, startdir):
     tests = {
         "BSD flags": has_lchflags,
-        "fuse": has_llfuse,
+        "fuse2": has_llfuse,
+        "fuse3": has_pyfuse3,
         "root": not fakeroot_detected(),
         "symlinks": are_symlinks_supported(),
         "hardlinks": are_hardlinks_supported(),
