@@ -12,7 +12,7 @@ from setuptools import Command
 
 
 def long_desc_from_readme():
-    with open('README.rst', 'r') as fd:
+    with open('README.rst') as fd:
         long_description = fd.read()
         # remove header, but have one \n before first headline
         start = long_description.find('What is BorgBackup?')
@@ -33,7 +33,7 @@ def format_metavar(option):
     elif option.nargs is None:
         return option.metavar
     else:
-        raise ValueError('Can\'t format metavar %s, unknown nargs %s!' % (option.metavar, option.nargs))
+        raise ValueError(f'Can\'t format metavar {option.metavar}, unknown nargs {option.nargs}!')
 
 
 class build_usage(Command):
@@ -367,7 +367,7 @@ class build_man(Command):
                 subparsers = [action for action in parser._actions if 'SubParsersAction' in str(action.__class__)][0]
                 for subcommand in subparsers.choices:
                     write('| borg', '[common options]', command, subcommand, '...')
-                    self.see_also.setdefault(command, []).append('%s-%s' % (command, subcommand))
+                    self.see_also.setdefault(command, []).append(f'{command}-{subcommand}')
             else:
                 if command == "borgfs":
                     write(command, end='')
