@@ -2,7 +2,11 @@ from contextlib import contextmanager
 import filecmp
 import functools
 import os
-import posix
+try:
+    import posix
+except ImportError:
+    posix = None
+
 import stat
 import sys
 import sysconfig
@@ -19,12 +23,10 @@ from .. import platform
 
 # Note: this is used by borg.selftest, do not use or import py.test functionality here.
 
-try:
-    import llfuse
-    # Does this version of llfuse support ns precision?
-    have_fuse_mtime_ns = hasattr(llfuse.EntryAttributes, 'st_mtime_ns')
-except ImportError:
-    have_fuse_mtime_ns = False
+from ..fuse_impl import llfuse, has_pyfuse3, has_llfuse
+
+# Does this version of llfuse support ns precision?
+have_fuse_mtime_ns = hasattr(llfuse.EntryAttributes, 'st_mtime_ns') if llfuse else False
 
 try:
     from pytest import raises
@@ -38,14 +40,8 @@ try:
 except OSError:
     has_lchflags = False
 
-try:
-    import llfuse
-    has_llfuse = True or llfuse  # avoids "unused import"
-except ImportError:
-    has_llfuse = False
-
 # The mtime get/set precision varies on different OS and Python versions
-if 'HAVE_FUTIMENS' in getattr(posix, '_have_functions', []):
+if posix and 'HAVE_FUTIMENS' in getattr(posix, '_have_functions', []):
     st_mtime_ns_round = 0
 elif 'HAVE_UTIMES' in sysconfig.get_config_vars():
     st_mtime_ns_round = -6
@@ -62,7 +58,7 @@ def unopened_tempfile():
         yield os.path.join(tempdir, "file")
 
 
-@functools.lru_cache()
+@functools.lru_cache
 def are_symlinks_supported():
     with unopened_tempfile() as filepath:
         try:
@@ -74,7 +70,7 @@ def are_symlinks_supported():
     return False
 
 
-@functools.lru_cache()
+@functools.lru_cache
 def are_hardlinks_supported():
     if not hasattr(os, 'link'):
         # some pythons do not have os.link
@@ -93,17 +89,22 @@ def are_hardlinks_supported():
     return False
 
 
-@functools.lru_cache()
+@functools.lru_cache
 def are_fifos_supported():
     with unopened_tempfile() as filepath:
         try:
             os.mkfifo(filepath)
             return True
         except OSError:
-            return False
+            pass
+        except NotImplementedError:
+            pass
+        except AttributeError:
+            pass
+        return False
 
 
-@functools.lru_cache()
+@functools.lru_cache
 def is_utime_fully_supported():
     with unopened_tempfile() as filepath:
         # Some filesystems (such as SSHFS) don't support utime on symlinks
@@ -118,10 +119,12 @@ def is_utime_fully_supported():
                 return True
         except OSError:
             pass
+        except NotImplementedError:
+            pass
         return False
 
 
-@functools.lru_cache()
+@functools.lru_cache
 def is_birthtime_fully_supported():
     if not hasattr(os.stat_result, 'st_birthtime'):
         return False
@@ -140,12 +143,14 @@ def is_birthtime_fully_supported():
                 return True
         except OSError:
             pass
+        except NotImplementedError:
+            pass
         return False
 
 
 def no_selinux(x):
     # selinux fails our FUSE tests, thus ignore selinux xattrs
-    SELINUX_KEY = 'security.selinux'
+    SELINUX_KEY = b'security.selinux'
     if isinstance(x, dict):
         return {k: v for k, v in x.items() if k != SELINUX_KEY}
     if isinstance(x, list):
@@ -159,7 +164,6 @@ class BaseTestCase(unittest.TestCase):
     assert_not_in = unittest.TestCase.assertNotIn
     assert_equal = unittest.TestCase.assertEqual
     assert_not_equal = unittest.TestCase.assertNotEqual
-    assert_true = unittest.TestCase.assertTrue
 
     if raises:
         assert_raises = staticmethod(raises)
@@ -168,15 +172,15 @@ class BaseTestCase(unittest.TestCase):
 
     @contextmanager
     def assert_creates_file(self, path):
-        self.assert_true(not os.path.exists(path), '{} should not exist'.format(path))
+        assert not os.path.exists(path), f'{path} should not exist'
         yield
-        self.assert_true(os.path.exists(path), '{} should exist'.format(path))
+        assert os.path.exists(path), f'{path} should exist'
 
     def assert_dirs_equal(self, dir1, dir2, **kwargs):
         diff = filecmp.dircmp(dir1, dir2)
         self._assert_dirs_equal_cmp(diff, **kwargs)
 
-    def _assert_dirs_equal_cmp(self, diff, ignore_bsdflags=False, ignore_xattrs=False, ignore_ns=False):
+    def _assert_dirs_equal_cmp(self, diff, ignore_flags=False, ignore_xattrs=False, ignore_ns=False):
         self.assert_equal(diff.left_only, [])
         self.assert_equal(diff.right_only, [])
         self.assert_equal(diff.diff_files, [])
@@ -196,7 +200,7 @@ class BaseTestCase(unittest.TestCase):
             d2 = [filename] + [getattr(s2, a) for a in attrs]
             d1.insert(1, oct(s1.st_mode))
             d2.insert(1, oct(s2.st_mode))
-            if not ignore_bsdflags:
+            if not ignore_flags:
                 d1.append(get_flags(path1, s1))
                 d2.append(get_flags(path2, s2))
             # ignore st_rdev if file is not a block/char device, fixes #203
@@ -222,33 +226,58 @@ class BaseTestCase(unittest.TestCase):
                 d2.append(no_selinux(get_all(path2, follow_symlinks=False)))
             self.assert_equal(d1, d2)
         for sub_diff in diff.subdirs.values():
-            self._assert_dirs_equal_cmp(sub_diff, ignore_bsdflags=ignore_bsdflags, ignore_xattrs=ignore_xattrs, ignore_ns=ignore_ns)
+            self._assert_dirs_equal_cmp(sub_diff, ignore_flags=ignore_flags, ignore_xattrs=ignore_xattrs, ignore_ns=ignore_ns)
 
     @contextmanager
-    def fuse_mount(self, location, mountpoint=None, *options, **kwargs):
+    def fuse_mount(self, location, mountpoint=None, *options, fork=True, os_fork=False, **kwargs):
+        # For a successful mount, `fork = True` is required for
+        # the borg mount daemon to work properly or the tests
+        # will just freeze. Therefore, if argument `fork` is not
+        # specified, the default value is `True`, regardless of
+        # `FORK_DEFAULT`. However, leaving the possibility to run
+        # the command with `fork = False` is still necessary for
+        # testing for mount failures, for example attempting to
+        # mount a read-only repo.
+        #    `os_fork = True` is needed for testing (the absence of)
+        # a race condition of the Lock during lock migration when
+        # borg mount (local repo) is daemonizing (#4953). This is another
+        # example where we need `fork = False`, because the test case
+        # needs an OS fork, not a spawning of the fuse mount.
+        # `fork = False` is implied if `os_fork = True`.
         if mountpoint is None:
             mountpoint = tempfile.mkdtemp()
         else:
             os.mkdir(mountpoint)
-        if 'fork' not in kwargs:
-            # For a successful mount, `fork = True` is required for
-            # the borg mount daemon to work properly or the tests
-            # will just freeze. Therefore, if argument `fork` is not
-            # specified, the default value is `True`, regardless of
-            # `FORK_DEFAULT`. However, leaving the possibilty to run
-            # the command with `fork = False` is still necessary for
-            # testing for mount failures, for example attempting to
-            # mount a read-only repo.
-            kwargs['fork'] = True
-        self.cmd('mount', location, mountpoint, *options, **kwargs)
-        if kwargs.get('exit_code', EXIT_SUCCESS) == EXIT_ERROR:
-            # If argument `exit_code = EXIT_ERROR`, then this call
-            # is testing the behavior of an unsuccessful mount and
-            # we must not continue, as there is no mount to work
-            # with. The test itself has already failed or succeeded
-            # with the call to `self.cmd`, above.
-            yield
-            return
+        args = ['mount', location, mountpoint] + list(options)
+        if os_fork:
+            # Do not spawn, but actually (OS) fork.
+            if os.fork() == 0:
+                # The child process.
+                # Decouple from parent and fork again.
+                # Otherwise, it becomes a zombie and pretends to be alive.
+                os.setsid()
+                if os.fork() > 0:
+                    os._exit(0)
+                # The grandchild process.
+                try:
+                    self.cmd(*args, fork=False, **kwargs)  # borg mount not spawning.
+                finally:
+                    # This should never be reached, since it daemonizes,
+                    # and the grandchild process exits before cmd() returns.
+                    # However, just in case...
+                    print('Fatal: borg mount did not daemonize properly. Force exiting.',
+                          file=sys.stderr, flush=True)
+                    os._exit(0)
+        else:
+            self.cmd(*args, fork=fork, **kwargs)
+            if kwargs.get('exit_code', EXIT_SUCCESS) == EXIT_ERROR:
+                # If argument `exit_code = EXIT_ERROR`, then this call
+                # is testing the behavior of an unsuccessful mount and
+                # we must not continue, as there is no mount to work
+                # with. The test itself has already failed or succeeded
+                # with the call to `self.cmd`, above.
+                yield
+                return
         self.wait_for_mountstate(mountpoint, mounted=True)
         yield
         umount(mountpoint)
@@ -264,7 +293,7 @@ class BaseTestCase(unittest.TestCase):
             if os.path.ismount(mountpoint) == mounted:
                 return
             time.sleep(0.1)
-        message = 'Waiting for %s of %s' % ('mount' if mounted else 'umount', mountpoint)
+        message = 'Waiting for {} of {}'.format('mount' if mounted else 'umount', mountpoint)
         raise TimeoutError(message)
 
     @contextmanager
